/*
 * vesalib.h
 *
 * VESALib header file
 */

#ifndef _VESALIB_H_
#define _VESALIB_H_


/*****************************************************************************
 Error Codes

 If a function succeeded, it will return 0, otherwise it returns one of the
 below defined error codes.
*****************************************************************************/

enum
{
    VESALIB_ERROR_NOVESA = 1,           // VESA 2.0 is not available
    VESALIB_ERROR_GETMODEINFOFAILED,    // could not get mode information
    VESALIB_ERROR_MODENOTAVAILABLE,     // mode is not available
    VESALIB_ERROR_MODESETFAILED,        // mode could not be set
    VESALIB_ERROR_LFBMAPPINGFAILED      // unable to map linear frame buffer
};


/*****************************************************************************
 Data Structures
*****************************************************************************/

/*
 * VESALIB_INFO: VESA 2.0 information structure
 */

typedef struct
{ 
    /*
     * All versions
     */

    INT8    VESASignature[4]    __attribute__ ((packed));
    UINT16  VESAVersion         __attribute__ ((packed));
    UINT32  OemStringPtr        __attribute__ ((packed));
    UINT32  Capabilities        __attribute__ ((packed));
    UINT32  VideoModePtr        __attribute__ ((packed));
    UINT16  TotalMemory         __attribute__ ((packed));

    /*
     * VESA 2
     */

    UINT16  OemSoftwareRev      __attribute__ ((packed));
    UINT32  OemVendorNamePtr    __attribute__ ((packed));
    UINT32  OemProductNamePtr   __attribute__ ((packed));
    UINT32  OemProductRevPtr    __attribute__ ((packed));
    UINT8   Reserved[222]       __attribute__ ((packed));
    UINT8   OemData[256]        __attribute__ ((packed));
} VESALIB_INFO;

/*
 * VESALIB_MODEINFO: VESA 2.1 mode information structure
 */

typedef struct
{
    /*
     * All versions
     */

    UINT16  ModeAttributes          __attribute__ ((packed));
    UINT8   WinAAttributes          __attribute__ ((packed));
    UINT8   WinBAttributes          __attribute__ ((packed));
    UINT16  WinGranularity          __attribute__ ((packed));
    UINT16  WinSize                 __attribute__ ((packed));
    UINT16  WinASegment             __attribute__ ((packed));
    UINT16  WinBSegment             __attribute__ ((packed));
    UINT32  WinFuncPtr              __attribute__ ((packed));
    UINT16  BytesPerScanLine        __attribute__ ((packed));
    UINT16  XResolution             __attribute__ ((packed));
    UINT16  YResolution             __attribute__ ((packed));
    UINT8   XCharSize               __attribute__ ((packed));
    UINT8   YCharSize               __attribute__ ((packed));
    UINT8   NumberOfPlanes          __attribute__ ((packed));
    UINT8   BitsPerPixel            __attribute__ ((packed));
    UINT8   NumberOfBanks           __attribute__ ((packed));
    UINT8   MemoryModel             __attribute__ ((packed));
    UINT8   BankSize                __attribute__ ((packed));
    UINT8   NumberOfImagePages      __attribute__ ((packed));
    UINT8   res1                    __attribute__ ((packed));
    UINT8   RedMaskSize             __attribute__ ((packed));
    UINT8   RedFieldPosition        __attribute__ ((packed));
    UINT8   GreenMaskSize           __attribute__ ((packed));
    UINT8   GreenFieldPosition      __attribute__ ((packed));
    UINT8   BlueMaskSize            __attribute__ ((packed));
    UINT8   BlueFieldPosition       __attribute__ ((packed));
    UINT8   RsvedMaskSize           __attribute__ ((packed));
    UINT8   RsvedFieldPosition      __attribute__ ((packed));
    UINT8   DirectColorModeInfo     __attribute__ ((packed));

    /*
     * VESA 2.0
     */

    UINT32  PhysBasePtr             __attribute__ ((packed));
    UINT32  OffScreenMemOffset      __attribute__ ((packed));
    UINT16  OffScreenMemSize        __attribute__ ((packed));

    /*
     * VESA 2.1
     */

    UINT16  LinbytesPerScanLine     __attribute__ ((packed));
    UINT8   BankNumberOfImagePages  __attribute__ ((packed));
    UINT8   LinNumberOfImagePages   __attribute__ ((packed));
    UINT8   LinRedMaskSize          __attribute__ ((packed));
    UINT8   LinRedFieldPosition     __attribute__ ((packed));
    UINT8   LingreenMaskSize        __attribute__ ((packed));
    UINT8   LinGreenFieldPosition   __attribute__ ((packed));
    UINT8   LinBlueMaskSize         __attribute__ ((packed));
    UINT8   LinBlueFieldPosition    __attribute__ ((packed));
    UINT8   LinRsvdMaskSize         __attribute__ ((packed));
    UINT8   LinRsvdFieldPosition    __attribute__ ((packed));
    UINT8   res2[194]               __attribute__ ((packed));
} VESALIB_MODEINFO;

/*
 * VESALIB_MODES: List of available modes
 */

typedef struct
{
    UINT16  mode_list[256];
    INT32   num_modes;      // number of modes in list
} VESALIB_MODES;

/*
 * VESALIB_LFBINFO: Linear frame buffer information
 */

typedef struct
{
    UINT32  pitch;              // total width of frame buffer
    UINT32  xres;               // horizontal resolution in pixels
    UINT32  yres;               // vertical resolution in pixels
    UINT32  bpp;                // bits per pixel
    UINT32  red_mask_size;      // number of bits in R component
    UINT32  red_field_pos;      // LSB number for R component
    UINT32  green_mask_size;    // number of bits in G component mask
    UINT32  green_field_pos;    // LSB number for G component
    UINT32  blue_mask_size;     // number of bits in B component
    UINT32  blue_field_pos;     // LSB number for B component
    UINT16  selector;           // protected mode selector for frame buffer
} VESALIB_LFBINFO;


/*****************************************************************************
 Functions

 See vesalib.c for information on what these functions do.
*****************************************************************************/

extern INT32            VESALib_CheckVESA();
extern INT32            VESALib_GetVESAInfo(VESALIB_INFO *);

extern void             VESALib_CreateModeList();
extern VESALIB_MODES    *VESALib_GetModeList();

extern INT32            VESALib_GetModeInfo(UINT16, VESALIB_MODEINFO *);
extern INT32            VESALib_FindMode(UINT16, UINT16, UINT8, UINT16 *);
extern INT32            VESALib_SetMode(UINT16);
extern UINT16           VESALib_GetCurrentMode();

extern INT32            VESALib_MapLinearFrameBuffer();
extern void             VESALib_GetLinearFrameBufferInfo(VESALIB_LFBINFO *);


#endif  /* _VESALIB_H_ */

